<?php

if (!defined('PHPWG_ROOT_PATH')) die('Hacking attempt!');

function image_resize_ext_im($result, $source_filepath, $destination_filepath, $max_width, $max_height, $quality)
{
  return pwg_image_resize_ext_im($result, $source_filepath, $destination_filepath, $max_width, $max_height, $quality, false);
}

function thumb_resize_ext_im($result, $source_filepath, $destination_filepath, $max_width, $max_height, $quality)
{
  return pwg_image_resize_ext_im($result, $source_filepath, $destination_filepath, $max_width, $max_height, $quality, true);
}

function pwg_image_resize_ext_im($result, $source_filepath, $destination_filepath, $max_width, $max_height, $quality, $thumb)
{
  global $conf;

  if ($result !== false)
  {
    //someone hooked us - so we skip
    return $result;
  }

  // extension of the picture filename
  $extension = strtolower(get_extension($source_filepath));
  if (!in_array($extension, array('jpg', 'jpeg', 'png')))
  {
    die('[Imagick] unsupported file extension');
  }

  $image = new Imagick($source_filepath);

  $rotation = get_rotation_angle($source_filepath);
  
  // width/height
  $source_width  = $image->getImageWidth();
  $source_height = $image->getImageHeight();

  if ($thumb and isset($conf['upload_form_thumb_square']) and $conf['upload_form_thumb_square'])
  {
    $coord = process_ratio($source_width, $source_height, $rotation);
    $image->cropImage($source_width, $source_height, $coord['x'], $coord['y']);
  }
  
  $resize_dimensions = get_resize_dimensions($source_width, $source_height, $max_width, $max_height, $rotation);

  // testing on height is useless in theory: if width is unchanged, there
  // should be no resize, because width/height ratio is not modified.
  if ($resize_dimensions['width'] == $source_width and $resize_dimensions['height'] == $source_height)
  {
    // the image doesn't need any resize! We just copy it to the destination
    copy($source_filepath, $destination_filepath);
    return true;
  }

  $image->setImageCompressionQuality($quality);
  $image->setInterlaceScheme(Imagick::INTERLACE_LINE);
  
  $image->resizeImage($resize_dimensions['width'], $resize_dimensions['height'], Imagick::FILTER_LANCZOS, 0.9);

  if (isset($rotation))
  {
    $image->rotateImage(new ImagickPixel(), -$rotation);
    $image->setImageOrientation(Imagick::ORIENTATION_TOPLEFT);
  }

  $image->writeImage($destination_filepath);
  $image->destroy();

  // everything should be OK if we are here!
  return true;
}

function get_rotation_angle($source_filepath)
{
  global $conf;

  if (!function_exists('exif_read_data'))
  {
    return null;
  }

  $rotation = null;
  
  $exif = exif_read_data($source_filepath);
  
  if (isset($exif['Orientation']) and preg_match('/^\s*(\d)/', $exif['Orientation'], $matches))
  {
    $orientation = $matches[1];
    if (in_array($orientation, array(3, 4)))
    {
      $rotation = 180;
    }
    elseif (in_array($orientation, array(5, 6)))
    {
      $rotation = 270;
    }
    elseif (in_array($orientation, array(7, 8)))
    {
      $rotation = 90;
    }
  }

  return $rotation;
}

function get_resize_dimensions($width, $height, $max_width, $max_height, $rotation=null)
{
  $rotate_for_dimensions = false;
  if (isset($rotation) and in_array(abs($rotation), array(90, 270)))
  {
    $rotate_for_dimensions = true;
  }

  if ($rotate_for_dimensions)
  {
    list($width, $height) = array($height, $width);
  }
  
  $ratio_width  = $width / $max_width;
  $ratio_height = $height / $max_height;
  
  // maximal size exceeded ?
  if ($ratio_width > 1 or $ratio_height > 1)
  {
    if ($ratio_width < $ratio_height)
    { 
      $destination_width = ceil($width / $ratio_height);
      $destination_height = $max_height;
    }
    else
    { 
      $destination_width = $max_width; 
      $destination_height = ceil($height / $ratio_width);
    }
  }

  if ($rotate_for_dimensions)
  {
    list($destination_width, $destination_height) = array($destination_height, $destination_width);
  }
  
  return array(
    'width' => $destination_width,
    'height'=> $destination_height,
    );
}

function process_ratio(&$source_width, &$source_height, $rotation=null)
{
  global $conf;

  if (!isset($conf['thumbnails_ratio']) or !preg_match('/^\d+:\d+$/', $conf['thumbnails_ratio']))
    $conf['thumbnails_ratio'] = '1:1';

  if (!isset($conf['thumbnails_ratio_orientation']))
    $conf['thumbnails_ratio_orientation'] = true;

  if (!$conf['thumbnails_ratio_orientation'] and $rotation)
    $conf['thumbnails_ratio_orientation'] = true;

  $x = 0;
  $y = 0;

  if ($source_width >= $source_height or !$conf['thumbnails_ratio_orientation'])
    list($widthRatio, $heightRatio) = explode(':', $conf['thumbnails_ratio']);
  else
    list($heightRatio, $widthRatio) = explode(':', $conf['thumbnails_ratio']);

  $img_ratio = $source_width / $source_height;
  $dest_ratio = $widthRatio / $heightRatio;

  if($dest_ratio > $img_ratio)
  {
    $destHeight = ceil(($source_width * $heightRatio) / $widthRatio);
    $y = ceil(($source_height - $destHeight) / 2 );
    $source_height = $destHeight ;
  }
  elseif ($dest_ratio < $img_ratio)
  {
    $destWidth = ceil(($source_height * $widthRatio) / $heightRatio);
    $x = ceil(($source_width - $destWidth) / 2 );
    $source_width = $destWidth ;
  }

  return array('x' => $x, 'y' => $y);
}

add_event_handler('upload_image_resize', 'image_resize_ext_im', 30, 6);
add_event_handler('upload_thumbnail_resize', 'thumb_resize_ext_im', 30, 6);

?>